part of masamune_mobile;

final localMediaProvider = ModelProvider.family<LocalMediaModel, String>(
    (_, path) => LocalMediaModel(path));

class LocalMediaModel extends Model<LocalMedia?> {
  LocalMediaModel(this.path) : super();

  final String path;

  final ImagePicker _picker = ImagePicker();

  Future<LocalMedia> _pickImage(ImageSource source) async {
    final pickedFile = await _picker.getImage(source: source);
    if (pickedFile == null || pickedFile.path.isEmpty) {
      throw Exception("The file was not found.");
    }
    final file = File(pickedFile.path);
    final copy = await file.copy(path);
    final media = LocalMedia(file: copy, assetType: AssetType.image);
    streamController.sink.add(media);
    return media;
  }

  Future<LocalMedia> _pickVideo(ImageSource source) async {
    final pickedFile = await _picker.getVideo(source: source);
    if (pickedFile == null || pickedFile.path.isEmpty) {
      throw Exception("The file was not found.");
    }
    final file = File(pickedFile.path);
    final copy = await file.copy(path);
    final media = LocalMedia(file: copy, assetType: AssetType.video);
    streamController.sink.add(media);
    return media;
  }

  /// Get image from camera or library.
  ///
  /// [source]: Media source.
  Future<LocalMedia> image({ImageSource source = ImageSource.camera}) async {
    assert(path.isNotEmpty, "The path is invalid.");
    final media = await _pickImage(source);
    streamController.sink.add(media);
    return media;
  }

  /// Get video from camera or library.
  ///
  /// [source]: Media source.
  Future<LocalMedia> video({ImageSource source = ImageSource.camera}) async {
    assert(path.isNotEmpty, "The path is invalid.");
    final media = await _pickVideo(source);
    streamController.sink.add(media);
    return media;
  }

  /// Delete the file.
  Future<void> delete() async {
    final file = File(path);
    await file.delete();
    streamController.sink.add(null);
  }

  /// Get image from camera or library by path.
  Future<LocalMedia> loadImage() async {
    assert(path.isNotEmpty, "The path is invalid.");
    final media = LocalMedia(file: File(path), assetType: AssetType.image);
    streamController.sink.add(media);
    return media;
  }

  /// Get video from camera or library by path.
  ///
  /// [path]: File path.
  /// [source]: Media source.
  /// [directoryType]: Directory destination to save.
  Future<LocalMedia> loadVideo() async {
    assert(path.isNotEmpty, "The path is invalid.");
    final media = LocalMedia(file: File(path), assetType: AssetType.image);
    streamController.sink.add(media);
    return media;
  }
}

class LocalMedia {
  const LocalMedia({required this.file, required this.assetType});
  final File file;
  final AssetType assetType;
}
